

C++++++++++++++++++++++++++++++++
C
      SUBROUTINE time_STATE(ET2,LIST,teph, vdc)
C
C++++++++++++++++++++++++++++++++
C
C THIS SUBROUTINE READS AND INTERPOLATES THE Irwin/Fukushima time EPHEMERIS FILE
C
C     CALLING SEQUENCE PARAMETERS:
C
C     INPUT:
C
C         ET2   DP 2-WORD JULIAN EPHEMERIS EPOCH AT WHICH INTERPOLATION
C               IS WANTED.  ANY COMBINATION OF ET2(1)+ET2(2) WHICH FALLS
C               WITHIN THE TIME SPAN ON THE FILE IS A PERMISSIBLE EPOCH.
C
C                A. FOR EASE IN PROGRAMMING, THE USER MAY PUT THE
C                   ENTIRE EPOCH IN ET2(1) AND SET ET2(2)=0.
C
C                B. FOR MAXIMUM INTERPOLATION ACCURACY, SET ET2(1) =
C                   THE MOST RECENT MIDNIGHT AT OR BEFORE INTERPOLATION
C                   EPOCH AND SET ET2(2) = FRACTIONAL PART OF A DAY
C                   ELAPSED BETWEEN ET2(1) AND EPOCH.
C
C                C. AS AN ALTERNATIVE, IT MAY PROVE CONVENIENT TO SET
C                   ET2(1) = SOME FIXED EPOCH, SUCH AS START OF INTEGRATION,
C                   AND ET2(2) = ELAPSED INTERVAL BETWEEN THEN AND EPOCH.
C
C        LIST(1)   = 1, time ephemeris (days) interpolated
C        LIST(1)   = 2, time ephemeris (days) and its derivative (dimensionless)
C          interpolated.
C        LIST(2)   = 1, v_E/c (dimensionless) interpolated
C        LIST(2)   = 2, v_E/c (dimensionless) and its derivative (days^{-1})
C          interpolated
C
C     OUTPUT:
C
C          teph(2)   DP vector THAT WILL CONTAIN REQUESTED INTERPOLATED
C               QUANTITIES.  teph(1) is the time ephemeris and teph(2) is
C               the time ephemeris derivative (if list(1) = 2)
C          vdc(3,2) DP vector that will contain requested interpolated
C               quantities.  vdc(1-3,1) is the v_E/c vector and
C               vdc(1-3,2) is the v_E/c derivative (i.e. acceleration) vector.
C
      IMPLICIT none

      SAVE
      integer*4 ncoeffs, maxncoeffs
C       accomodates te406 (with v_E/c) and all other time ephemerides
      parameter(maxncoeffs = 322)
      real*8 ET2(2),teph(2), vdc(3,2), T(2),PJD(4),BUF(maxncoeffs),
     &  SS(3), CVAL(2), s

      INTEGER *4 LIST(2), ncon, ipt(3,2)

      LOGICAL FIRST
      DATA FIRST/.TRUE./

      CHARACTER*6 TTL(14,3),CNAM(2)
      CHARACTER*80 NAMFIL
      COMMON/time_EPHHDR/CVAL,SS,NCON,IPT
      COMMON/time_CHRHDR/CNAM,TTL
      integer*4 nrecl, ksize, nrfile, nr, nrl, irecsz, k

C
C       ENTRY POINT - 1ST TIME IN, GET POINTER DATA, ETC., FROM EPH FILE
C
      IF(FIRST) THEN
        FIRST=.FALSE.

C ************************************************************************
C ************************************************************************

C THE USER MUST SELECT ONE OF THE FOLLOWING BY DELETING THE 'C' IN COLUMN 1

C ************************************************************************

C        CALL FSIZER1(NRECL,KSIZE,NRFILE,NAMFIL)
        CALL time_FSIZER2(NRECL,KSIZE,NRFILE,NAMFIL)
C        CALL FSIZER3(NRECL,KSIZE,NRFILE,NAMFIL)

      IF(NRECL .EQ. 0) WRITE(*,*)'  ***** FSIZER IS NOT WORKING *****'

C ************************************************************************
C ************************************************************************

      IRECSZ=NRECL*KSIZE
      NCOEFFS=KSIZE/2
      if(ncoeffs.gt.maxncoeffs)
     &  stop 'bad time ephemeris buffer length'

        OPEN(NRFILE,
     *    FILE=NAMFIL,
     *       ACCESS='DIRECT',
     *       FORM='UNFORMATTED',
     *       RECL=IRECSZ,
     *       STATUS='OLD')

      READ(NRFILE,REC=1)TTL,CNAM,SS,NCON,ipt

      READ(NRFILE,REC=2)CVAL

      NRL=0

      ENDIF


C       ********** MAIN ENTRY POINT **********


      IF(ET2(1) .EQ. 0.D0) RETURN
      if(list(2).gt.0.and.ipt(2,2).eq.0)
     &  stop 'error: request for v_E/c data from ephemeris without it'

      S=ET2(1)-.5D0
      CALL SPLIT(S,PJD(1))
      CALL SPLIT(ET2(2),PJD(3))
      PJD(1)=PJD(1)+PJD(3)+.5D0
      PJD(2)=PJD(2)+PJD(4)
      CALL SPLIT(PJD(2),PJD(3))
      PJD(1)=PJD(1)+PJD(3)

C       ERROR RETURN FOR EPOCH OUT OF RANGE

      IF(PJD(1)+PJD(4).LT.SS(1) .OR. PJD(1)+PJD(4).GT.SS(2)) GO TO 98

C       CALCULATE RECORD # AND RELATIVE TIME IN INTERVAL

      NR=IDINT((PJD(1)-SS(1))/SS(3))+3
      IF(PJD(1).EQ.SS(2)) NR=NR-1
      T(1)=((PJD(1)-(DBLE(NR-3)*SS(3)+SS(1)))+PJD(4))/SS(3)
      t(2) = ss(3)

C       READ CORRECT RECORD IF NOT IN CORE

      IF(NR.NE.NRL) THEN
        NRL=NR
        READ(NRFILE,REC=NR,ERR=99)(BUF(K),K=1,NCOEFFS)
      ENDIF

C   INTERPOLATE time ephemeris

      if(list(1).gt.0) then
        CALL INTERP(BUF(IPT(1,1)),T,IPT(2,1),1,IPT(3,1),list(1),teph)
      elseif(list(2).gt.0) then
        CALL INTERP(BUF(IPT(1,2)),T,IPT(2,2),3,IPT(3,2),list(2),vdc)
      endif
      RETURN

  98  WRITE(*,198)ET2(1)+ET2(2),SS(1),SS(2)
 198  format(' ***  Requested JED,',f12.2,
     * ' not within ephemeris limits,',2f12.2,'  ***')

      stop

   99 WRITE(*,'(2F12.2,A80)')ET2,'ERROR RETURN IN STATE'

      STOP

      END
